<?php
// $LastChangedDate: 2012-09-17 19:12:38 +0200 (Mon, 17 Sep 2012) $
// $Rev: 1843 $

class IPv4
{
	private $ip_long;
	public $ip;
	public $cidr;

	function __construct($ip,$cidr)
	{
		$this->ip = $ip;
		$this->ip_long = ip2long($ip);
		$this->cidr = $cidr;
	}

	function __toString(){
		$methods = get_class_methods($this);	

		foreach($methods as $meth){
			if(substr($meth, 0, 2) != '__' && $meth != 'mask2cidr' && $meth != 'cidr2mask'){
				$r[] = $this->$meth();
			}
		}

		return json_encode($r);
	}

	static function mask2cidr($mask)
	{
		$mask = ip2long($mask);
		$base = ((1<<32)-1);
		return 32-log(($mask ^ $base)+1,2);
	}

	static function cidr2mask($cidr)
	{
		$mask = ip2long('255.255.255.255');
		$base = ((1<<$cidr)-1);
		return 32-log(($mask ^ $base)+1,2);
	}

	function address(){
		return $this->ip;
	}

	function cidr() {
		return $this->cidr;
	}

	function netmask()
	{
		$netmask = ((1<<32) -1) << (32-$this->cidr);
		return long2ip($netmask);
	}	

	function network()
	{
		$network = $this->ip_long & (ip2long($this->netmask()));
		return long2ip($network);
	}

	function broadcast()
	{
		$broadcast = ip2long($this->network()) | (~(ip2long($this->netmask())));
		return long2ip($broadcast);
	}

	function wildcard()
	{
		$inverse = ~(((1<<32) -1) << (32-$this->cidr));
		return long2ip($inverse);
	}

	function availablehosts()
	{
		$hosts = (ip2long($this->broadcast()) - ip2long($this->network())) -1;
		return $hosts;
	}

	function availablenetworks()
	{
		return pow(2, 24)/($this->availablehosts()+2);
	}

	function isipin($ip)
	{
		$lnet=ip2long($this->network());
		$lip=ip2long($ip);
		$binnet=str_pad( decbin($lnet),32,"0",STR_PAD_LEFT );
		$firstpart=substr($binnet,0,$this->mask2cidr($this->netmask()));
		$binip=str_pad( decbin($lip),32,"0",STR_PAD_LEFT );
		$firstip=substr($binip,0,$this->mask2cidr($this->netmask()));
		return(strcmp($firstpart,$firstip)==0);
	}

	function increment($start,$end)
	{
		$start_arr = explode('.',$start);
		$end_arr = explode('.',$end);

		while($start_arr <= $end_arr)
		{
			$start_arr[3]++;
			if($start_arr[3] == 256)
			{
				$start_arr[3] = 0;
				$start_arr[2]++;
				if($start_arr[2] == 256)
				{
					$start_arr[2] = 0;
					$start_arr[1]++;
					if($start_arr[1] == 256)
					{
						$start_arr[1] = 0;
						$start_arr[0]++;
					} else {
						return implode('.',$start_arr);
					}
				} else {
					return implode('.',$start_arr);
				}
			} else {
				return implode('.',$start_arr);
			}
		}
	}

	function is_ip4($ip)
	{
		if ( preg_match('/^([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])$/',$ip))
		{
                	return true;                                                                                                    
		} else {
			return false;
		}
	}

	function is_net4($net)
	{
        	if ( preg_match('/^([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])\.([0-9]{1,2}|[01][0-9]{2}|2[0-4][0-9]|25[0-5])\/[0-9]{1,2}$/', $net ))
		{
			return true;
		} else {
			return false;
		}
	}

	function reverse_ip4($ip)
	{
	        if ( is_ip( $ip ) )
		{
			$iparr = explode(".", $ip);
			$rip = $iparr[3] . "." . $iparr[2] . "." . $iparr[1] . "." . $iparr[0];
			return $rip;
		} else {
			return false;
		}
	}

}
?>
